/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.common;

import java.io.*;
import java.util.Vector;

import sync4j.syncclient.common.VectorTools;
/**
 * This class supplies some methods of usefullness for the
 * management of files and directories.
 * <p>In particular it supplies methods for the creation and the reading of
 * file and for the removal of a directory and of its everything content
 *
 * @version  $Id: FileSystemTools.java,v 1.2.2.1 2005/01/27 11:00:57 fabius Exp $
 * @author Stefano Nichele
 */

public class FileSystemTools {

    // -----------------------------------------------------------Public methods

    /**
     * Delete a directory, his subdirectory and all files.
     * @param directoryName the name of the directory to remove.
     * @throws Exception if an error occurs.
     */
    public static void removeDirectoryTree(String directoryName)
            throws Exception {

        File directory = new File(directoryName);

        if (!directory.exists()) {
            return;
        }

        if (!directory.isDirectory()) {
            throw new Exception("'" + directory + "' is not a directory");
        }

        String[] fileList = directory.list();
        int numFile = fileList.length;
        boolean fileDeleted = false;
        File f = null;
        for (int i=0; i<numFile; i++) {
            f = new File(directoryName + File.separator +  fileList[i]);
            if (f.isDirectory()) {
                removeDirectoryTree(f.getPath());
            } else {
                fileDeleted = f.delete();
            }
        }
        fileDeleted = directory.delete();
    }

    /**
     * Create a new file in the specified directory using
     * the given file name and content.
     * <p>If the directory it does not exist it will be created.</p>
     * <p>If already exists a file with the same name, this has deleted before
     * create the new file.</p>
     * @param directoryName the directory in which the file must be created.
     * @param fileName the name of the file
     * @param content the content of the file.
     * @throws Exception if an error occurs during creation.
     */
    public static void createFile(String directoryName,
                                  String fileName, byte[] content)
            throws Exception {

        int size = content.length;

        File fileOut = new File(directoryName +
                File.separator + fileName);

        fileOut.mkdirs();

        if (fileOut.exists()) {
            fileOut.delete();
        }

        FileOutputStream fout = new FileOutputStream(fileOut);

        fout.write(content);
        fout.close();

    }


    /**
     * Create a new file in the specified directory using
     * the given file name and byteStream.
     * <p>If the directory it does not exist it will be created.</p>
     * <p>If already exists a file with the same name, this has deleted before
     * create the new file.</p>
     * @param directoryName the directory in which the file must be created.
     * @param fileName the name of the file
     * @param byteStream the content of the file.
     * @throws Exception if an error occurs during creation.
     */
    public static void createFile(String directoryName,
                                  String fileName, ByteArrayOutputStream byteStream)
            throws Exception {

        File fileOut = new File(directoryName +
                File.separator + fileName);

        fileOut.mkdirs();

        if (fileOut.exists()) {
            fileOut.delete();
        }

        FileOutputStream fout = new FileOutputStream(fileOut);
        byteStream.writeTo(fout);
        fout.close();

    }

    /**
     * Returns the content of the file specified.
     * @param fileName the name of the file.
     * @return the content of the file.
     * @throws IOException if the file not exits or if an I/O error occurs.
     */
    public static byte[] getFile(String fileName) throws IOException {
        File file = new File(fileName);
        int dim = (int)file.length();
        byte[] content = new byte[dim];

        BufferedInputStream stream =
                new BufferedInputStream(new FileInputStream(fileName));
        stream.read(content);
        stream.close();
        return content;
    }

    /**
     * Returns all files in the given directory filtering on the given extension.
     * Pathnames are returned relative to <i>directory</i>
     *
     * @param directory the directory to list
     * @param extension select only the files with this extension
     *
     * @return String[] with the selected filenames. An zero-length array if
     *                  no files are selected
     */
    public static String[] getAllFiles(String  directory,
                                       String  extension) {

        ExtensionFilter filter = new ExtensionFilter(extension);

        Vector fileList = new Vector();

        String[] files = new File(directory).list(filter);

        if ((files == null) || (files.length == 0)) {
            return new String[0];
        }

        File f;
        String[] children;
        for (int i=0; i<files.length; ++i) {
            f = new File(directory, files[i]);
            if (f.isDirectory()) {
                children = getAllFiles(f.getPath(),  extension);
                VectorTools.add(fileList, children);
            }
            //
            // we have to exclude directories
            //
            if (f.isFile()){
                fileList.addElement(f.getPath());
            }
        }

        return VectorTools.toStringArray(fileList);
    }
}